@@ -1,3 +1,6 @@
+{-# LANGUAGE DeriveAnyClass #-}
+{-# LANGUAGE RecordWildCards #-}
+
 -- SPDX-FileCopyrightText: Copyright (c) 2025 Objectionary.com
 -- SPDX-License-Identifier: MIT
 
@@ -7,21 +10,54 @@
 module Builder
   ( buildExpressions,
     buildExpression,
-    buildTermFromFunction,
+    buildExpressionThrows,
     buildAttribute,
+    buildAttributeThrows,
     buildBinding,
+    buildBindingThrows,
+    buildBytes,
+    buildBytesThrows,
     contextualize,
-    Term (..),
+    BuildException (..),
   )
 where
 
 import Ast
+import Control.Exception (Exception, throwIO)
 import qualified Data.Map.Strict as Map
 import Matcher
-import Pretty (prettyAttribute)
+import Pretty (prettyAttribute, prettyBinding, prettyExpression, prettySubst, prettyBytes)
+import Text.Printf (printf)
 import Yaml (ExtraArgument (..))
 
-data Term = TeExpression Expression | TeAttribute Attribute
+data BuildException
+  = CouldNotBuildExpression {_expr :: Expression, _subst :: Subst}
+  | CouldNotBuildAttribute {_attr :: Attribute, _subst :: Subst}
+  | CouldNotBuildBinding {_bd :: Binding, _subst :: Subst}
+  | CouldNotBuildBytes {_bts :: Bytes, _subst :: Subst}
+  deriving (Exception)
+
+instance Show BuildException where
+  show CouldNotBuildExpression {..} =
+    printf
+      "Couldn't build given expression with provided substitutions\n--Expression: %s\n--Substitutions: %s"
+      (prettyExpression _expr)
+      (prettySubst _subst)
+  show CouldNotBuildAttribute {..} =
+    printf
+      "Couldn't build given attribute with provided substitutions\n--Attribute: %s\n--Substitutions: %s"
+      (prettyAttribute _attr)
+      (prettySubst _subst)
+  show CouldNotBuildBinding {..} =
+    printf
+      "Couldn't build given binding with provided substitutions\n--Binding: %s\n--Substitutions: %s"
+      (prettyBinding _bd)
+      (prettySubst _subst)
+  show CouldNotBuildBytes {..} =
+    printf
+      "Couldn't build given bytes with provided substitutions\n--Bytes: %s\n--Substitutions: %s"
+      (prettyBytes _bts)
+      (prettySubst _subst)
 
 contextualize :: Expression -> Expression -> Program -> Expression
 contextualize ExGlobal _ (Program expr) = expr
@@ -40,6 +76,12 @@ buildAttribute (AtMeta meta) (Subst mp) = case Map.lookup meta mp of
   _ -> Nothing
 buildAttribute attr _ = Just attr
 
+buildBytes :: Bytes -> Subst -> Maybe Bytes
+buildBytes (BtMeta meta) (Subst mp) = case Map.lookup meta mp of
+  Just (MvBytes bytes) -> Just bytes
+  _ -> Nothing
+buildBytes bts _ = Just bts
+
 -- Build binding
 -- The function returns [Binding] because the BiMeta is always attached
 -- to the list of bindings
@@ -54,9 +96,9 @@ buildBinding (BiVoid attr) subst = do
 buildBinding (BiMeta meta) (Subst mp) = case Map.lookup meta mp of
   Just (MvBindings bds) -> Just bds
   _ -> Nothing
-buildBinding (BiMetaDelta meta) (Subst mp) = case Map.lookup meta mp of
-  Just (MvBytes bytes) -> Just [BiDelta bytes]
-  _ -> Nothing
+buildBinding (BiDelta bytes) subst = do
+  bts <- buildBytes bytes subst
+  Just [BiDelta bts]
 buildBinding (BiMetaLambda meta) (Subst mp) = case Map.lookup meta mp of
   Just (MvFunction func) -> Just [BiLambda func]
   _ -> Nothing
@@ -105,47 +147,26 @@ buildExpression (ExMetaTail expr meta) subst = do
     _ -> Nothing
 buildExpression expr _ = Just (expr, defaultScope)
 
-buildTermFromFunction :: String -> [ExtraArgument] -> Subst -> Program -> Maybe Term
-buildTermFromFunction "contextualize" [ArgExpression expr, ArgExpression context] subst prog = do
-  (expr', _) <- buildExpression expr subst
-  (context', _) <- buildExpression context subst
-  return (TeExpression (contextualize expr' context' prog))
-buildTermFromFunction "scope" [ArgExpression expr] subst prog = do
-  (expr', scope) <- buildExpression expr subst
-  return (TeExpression scope)
-buildTermFromFunction "random-tau" args subst _ = do
-  attrs <- argsToAttrs args
-  return (TeAttribute (AtLabel (randomTau 0 attrs)))
-  where
-    argsToAttrs :: [ExtraArgument] -> Maybe [String]
-    argsToAttrs [] = Just []
-    argsToAttrs (arg : rest) = case arg of
-      ArgExpression _ -> argsToAttrs rest
-      ArgAttribute attr -> do
-        attr' <- buildAttribute attr subst
-        rest' <- argsToAttrs rest
-        Just (prettyAttribute attr' : rest')
-      ArgBinding bd -> do
-        bds <- buildBinding bd subst
-        rest' <- argsToAttrs rest
-        Just (attrsFromBindings bds ++ rest')
-    attrsFromBindings :: [Binding] -> [String]
-    attrsFromBindings [] = []
-    attrsFromBindings (bd : bds) =
-      let attr = case bd of
-            BiTau attr _ -> attr
-            BiDelta _ -> AtDelta
-            BiLambda _ -> AtLambda
-            BiVoid attr -> attr
-       in prettyAttribute attr : attrsFromBindings bds
-    randomTau :: Integer -> [String] -> String
-    randomTau _ [] = cactoos
-    randomTau idx attrs =
-      let tau = if idx == 0 then cactoos else cactoos ++ show idx
-       in if tau `elem` attrs then randomTau (idx + 1) attrs else tau
-    cactoos = "a🌵"
-buildTermFromFunction _ _ _ _ = Nothing
+buildBytesThrows :: Bytes -> Subst -> IO Bytes
+buildBytesThrows bytes subst = case buildBytes bytes subst of
+  Just bts -> pure bts
+  _ -> throwIO (CouldNotBuildBytes bytes subst)
+
+buildBindingThrows :: Binding -> Subst -> IO [Binding]
+buildBindingThrows bd subst = case buildBinding bd subst of
+  Just bds -> pure bds
+  _ -> throwIO (CouldNotBuildBinding bd subst)
+
+buildAttributeThrows :: Attribute -> Subst -> IO Attribute
+buildAttributeThrows attr subst = case buildAttribute attr subst of
+  Just attr' -> pure attr'
+  _ -> throwIO (CouldNotBuildAttribute attr subst)
+
+buildExpressionThrows :: Expression -> Subst -> IO (Expression, Expression)
+buildExpressionThrows expr subst = case buildExpression expr subst of
+  Just built -> pure built
+  _ -> throwIO (CouldNotBuildExpression expr subst)
 
 -- Build a several expression from one expression and several substitutions
-buildExpressions :: Expression -> [Subst] -> Maybe [(Expression, Expression)]
-buildExpressions expr = traverse (buildExpression expr)
+buildExpressions :: Expression -> [Subst] -> IO [(Expression, Expression)]
+buildExpressions expr = traverse (buildExpressionThrows expr)
