@@ -178,6 +178,18 @@ spec = do
           ["rewrite", "--nothing", "--output=xmir", "--omit-listing"]
           ["<?xml version=\"1.0\" encoding=\"UTF-8\"?>", "<object", "<listing>4 lines of phi</listing>", "  <o base=\"Q.y\" name=\"x\"/>"]
 
+    it "fails with --nothing and --must=2" $
+      withStdin "Q -> [[ ]]" $
+        testCLIFailed ["rewrite", "--nothing", "--must=2"] "it's expected exactly 2 rewriting cycles happened, but rewriting stopped after 1"
+
+    it "does not fail with --nothing and --must" $
+      withStdin "Q -> [[ ]]" $
+        testCLI ["rewrite", "--nothing", "--must"] ["Φ ↦ ⟦ ρ ↦ ∅ ⟧"]
+
+    it "fails with --normalize and --must" $
+      withStdin "Q -> [[ x -> [[ y -> 5 ]].y ]].x" $
+        testCLIFailed ["rewrite", "--normalize", "--must"] "it's expected exactly 1 rewriting cycles happened, but rewriting is still going"
+
   describe "dataize" $ do
     it "dataizes simple program" $
       withStdin "Q -> [[ D> 01- ]]" $
